/*
 * main.cpp
 *
 *  Created on: Feb 6, 2010
 *      Author: gregor
 */

#include <iostream>
#include <sstream>
#include <cstdlib>
#ifdef LINUX
#include <unistd.h>
#endif

#include "GL/glew.h"
#include "GL/glfw.h"

#include "GLTB/exception.h"

#include "GL3Support/framebuffer.h"
#include "GL3Support/framebuffermanager.h"
#include "GL3Support/resourcemanager.h"
#include "GL3Support/shader.h"
#include "GL3Support/texture.h"
#include "GL3Support/vertexbuffer.h"

const int width=1920;
const int height=1200;

int main(int argc, char *argv[])
{
	try {
		glfwInit();

		glfwOpenWindow(width,height,8,8,8,8,24,8,GLFW_FULLSCREEN);
//		glfwOpenWindow(width,height,8,8,8,8,24,8,GLFW_WINDOW);

		std::cout << "OpenGL vendor: " << glGetString(GL_VENDOR) << std::endl;
		std::cout << "OpenGL renderer: " << glGetString(GL_RENDERER) << std::endl;
		std::cout << "OpenGL version: " << glGetString(GL_VERSION) << std::endl;
		std::cout << "available extensions: " << glGetString(GL_EXTENSIONS) << std::endl;

		int maxUniforms;
		glGetIntegerv(GL_MAX_VERTEX_UNIFORM_COMPONENTS,&maxUniforms);
		std::cout << "max. vertex shader uniforms: " << maxUniforms << std::endl;
		glGetIntegerv(GL_MAX_FRAGMENT_UNIFORM_COMPONENTS,&maxUniforms);
		std::cout << "max. fragment shader uniforms: " << maxUniforms << std::endl;

		glewInit();

		std::cout << "GLEW initialized" << std::endl;

		gltb::RefPtr<GL3Support::Shader> brainShader=GL3Support::ResourceManager::getInstance()->createShader();
		brainShader->loadVertexShader("glsl/brain_vertex.glsl");
		brainShader->loadFragmentShader("glsl/brain_fragment.glsl");
		brainShader->prepare();

		gltb::RefPtr<GL3Support::Shader> resultShader=GL3Support::ResourceManager::getInstance()->createShader();
		resultShader->loadVertexShader("glsl/result_vertex.glsl");
		resultShader->loadFragmentShader("glsl/result_fragment.glsl");
		resultShader->prepare();

		std::cout << "shaders compiled" << std::endl;

		gltb::RefPtr<GL3Support::Framebuffer> brainBuffer=GL3Support::FramebufferManager::getInstance()->createFramebuffer(width,height,"brainBuffer");

		gltb::RefPtr<GL3Support::Texture> brainTexture=GL3Support::ResourceManager::getInstance()->createTexture(GL3Support::Texture::TwoDRect,GL3Support::Texture::FloatR,width,height,"brainTex");
		brainBuffer->attachTexture(GL3Support::Framebuffer::ColorAttachment,brainTexture);

		brainBuffer->activate();
		if(!brainBuffer->isValid()) {
			throw gltb::Exception("brain buffer invalid","main()");
		}
		brainBuffer->deactivate();

		brainTexture->readBack();
		float *brain=(float*)brainTexture->getData();
		for(unsigned int i=0;i<width*height;i++) {
//			brain[i]=rand()%3;
			brain[i]=0;
		}
		for(unsigned int i=width/4;i<3*width/4;i++) {
			for(unsigned int j=width/4;j<3*height/4;j++) {
				brain[j*width+i]=rand()%3;
			}
		}
		brainTexture->reupload();

		gltb::RefPtr<GL3Support::Framebuffer> brainBufferTarget=GL3Support::FramebufferManager::getInstance()->createFramebuffer(width,height,"brainBufferTarget");

		gltb::RefPtr<GL3Support::Texture> brainTextureTarget=GL3Support::ResourceManager::getInstance()->createTexture(GL3Support::Texture::TwoDRect,GL3Support::Texture::FloatR,width,height,"brainTexTarget");
		brainBufferTarget->attachTexture(GL3Support::Framebuffer::ColorAttachment,brainTextureTarget);

		brainBufferTarget->activate();
		if(!brainBufferTarget->isValid()) {
			throw gltb::Exception("brain target buffer invalid","main()");
		}
		brainBufferTarget->deactivate();

		int running=GL_TRUE;
		double t, t0;
		int frames=0;
		t0=glfwGetTime();
		while(running) {
			glViewport(0,0,width,height);
			glClearColor(0.0f,0.0f,0.0f,0.0f);
//			glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
			glClear(GL_DEPTH_BUFFER_BIT);
			glMatrixMode(GL_MODELVIEW);
			glLoadIdentity();
			glMatrixMode(GL_PROJECTION);
			glLoadIdentity();
			glOrtho(0,width,0,height,-10,10);

			brainBufferTarget->activate();

			GL3Support::Shader::Uniform param;
			param.type=GL3Support::Shader::Sampler2D;
			param.generatedTexture=false;
			param.stringValue=brainTexture->getName();
			brainShader->setUniform("brain",param);

			brainShader->activate();
			// render full screen quad to other framebuffer to simulate automaton step
			glBegin(GL_QUADS);
			glTexCoord2d(0.0, 0.0);
			glVertex3d(0,0,0);
			glTexCoord2d(0.0, height);
			glVertex3d(0,height,0);
			glTexCoord2d(width, height);
			glVertex3d(width,height,0);
			glTexCoord2d(width, 0.0);
			glVertex3d(width,0,0);
			glEnd();
			brainShader->deactivate();

			brainBufferTarget->deactivate();

//			brainTexture->readBack();
//			brain=(float*)brainTexture->getData();
//			dump=fopen("brian3.dat","wb");
//			fwrite(brain,width*height,sizeof(float),dump);
//			fclose(dump);
//			brainTexture->reupload();

			param.type=GL3Support::Shader::Sampler2D;
			param.generatedTexture=false;
			param.stringValue=brainTextureTarget->getName();
//			param.stringValues[0]=brainTexture->getName();
			resultShader->setUniform("brain",param);

			// render full screen quad to screen
			resultShader->activate();
			glBegin(GL_QUADS);
			glTexCoord2d(0.0, 0.0);
			glVertex3d(0,0,0);
			glTexCoord2d(0.0, height);
			glVertex3d(0,height,0);
			glTexCoord2d(width, height);
			glVertex3d(width,height,0);
			glTexCoord2d(width, 0.0);
			glVertex3d(width,0,0);
			glEnd();
			resultShader->deactivate();

			gltb::RefPtr<GL3Support::Framebuffer> tempBuffer=brainBuffer;
			brainBuffer=brainBufferTarget;
			brainBufferTarget=tempBuffer;

			gltb::RefPtr<GL3Support::Texture> tempTexture=brainTexture;
			brainTexture=brainTextureTarget;
			brainTextureTarget=tempTexture;

			// Calculate and display FPS (frames per second)
			t=glfwGetTime();
			if( (t-t0) > 1.0 || frames == 0 )
			{
				double fps = (double)frames / (t-t0);
				std::ostringstream title;
				title << "FPS: " << fps;
				std::cout << "FPS: " << fps << std::endl;
				glfwSetWindowTitle( title.str().c_str() );
				t0 = t;
				frames = 0;
			}
			frames ++;

			// Check if the ESC key was pressed or the window was closed
			running = !glfwGetKey( GLFW_KEY_ESC ) &&
					glfwGetWindowParam( GLFW_OPENED );

			glfwSwapBuffers();

			usleep(16000);
		}


		glfwTerminate();

	} catch(gltb::Error e) {
		std::cerr << "Error: " << e.getDescription() << std::endl;
		std::cerr << "In: " << e.getLocation() << std::endl;
	} catch(gltb::Exception e) {
		std::cerr << "Exception: " << e.getDescription() << std::endl;
		std::cerr << "In: " << e.getLocation() << std::endl;
	}
	return 0;
}
