/*
 * main.cpp
 *
 *  Created on: Jun 19, 2009
 *      Author: muecklgr_local
 */

#include <iostream>

#include "GL/glew.h"
#include "GL/glfw.h"

#include "GLTB/exception.h"

#include "GL3Support/glhelper.h"
#include "GL3Support/resourcemanager.h"
#include "GL3Support/shader.h"

int main(int argc, char *argv[])
{
	try {
		if(argc!=3) {
			std::cerr << "usage: " << argv[0] << " [vertex shader file] [fragment shader file]" << std::endl;
			return -1;
		}

		std::string vertexShaderFile=argv[1];
		std::string fragmentShaderFile=argv[2];

		glfwInit();

		glfwOpenWindowHint(GLFW_OPENGL_DEBUG_CONTEXT,1);
		glfwOpenWindow(320,200,8,8,8,8,24,8,GLFW_WINDOW);

		std::cout << "OpenGL vendor: " << glGetString(GL_VENDOR) << std::endl;
		std::cout << "OpenGL renderer: " << glGetString(GL_RENDERER) << std::endl;
		std::cout << "OpenGL version: " << glGetString(GL_VERSION) << std::endl;
		std::cout << "available extensions: " << glGetString(GL_EXTENSIONS) << std::endl;

		glewInit();

		GL3Support::GLHelper::enableDebugLogging();

		gltb::RefPtr<GL3Support::Shader> shader=GL3Support::ResourceManager::getInstance()->createShader();

		shader->loadVertexShader(vertexShaderFile);
		shader->loadFragmentShader(fragmentShaderFile);

		shader->prepare();

		shader->printParameterInfo();

		glfwTerminate();

	} catch(gltb::Error e) {
		std::cerr << "Error: " << e.getDescription() << std::endl;
		std::cerr << "In: " << e.getLocation() << std::endl;
	} catch(gltb::Exception e) {
		std::cerr << "Exception: " << e.getDescription() << std::endl;
		std::cerr << "In: " << e.getLocation() << std::endl;
	}
	return 0;
}
