#ifndef CAMERA_H_
#define CAMERA_H_

#include "GL3Support/transformable.h"

namespace GL3Support {
	class Camera : public Transformable {
		public:
			enum ProjectionType {
				Perspective,
				Orthographic
			};
			
			Camera();
			virtual ~Camera();

			void setPerspectiveProjection(float fieldOfView, float aspectRatio, float nearClip, float farClip);
			void getPerspectiveProjection(float &fieldOfView, float  &aspectRatio, float &nearClip, float &farClip);
			void setOrthographicProjection(float left, float right, float bottom, float top, float near, float far);
			void getOrthographicProjection(float &left, float &right, float &bottom, float &top, float &near, float &far);

			ProjectionType getProjectionType();
			glm::mat4 getProjectionMatrix();

			void setPositionLookAt(glm::vec3 position, glm::vec3 front, glm::vec3 up);

			// reimplemented from Transformable:
			virtual void setTransformationMatrix(glm::mat4 matrix);
			virtual glm::mat4 getTransformationMatrix();

			glm::vec3 getFront();
			glm::vec3 getUp();

		private:
			ProjectionType projectionType;

			float fieldOfView, aspectRatio, nearClip, farClip;
			float left, right, bottom, top, near, far;

			glm::mat4 projectionMatrix;
			glm::mat4 transformationMatrix;
	};
}

#endif /*CAMERA_H_*/
