#ifndef FRAMEBUFFER_H_
#define FRAMEBUFFER_H_

#include <vector>

#include "GL/glew.h"

#include "GLTB/refptr.h"
#include "GLTB/referencedobject.h"

#include "GL3Support/texture.h"

namespace GL3Support {
	class Texture;

	class Framebuffer : public gltb::ReferencedObject {
	public:
		enum Attachment {
			ColorAttachment,
			DepthAttachment,
			StencilAttachment
		};

		Framebuffer(GLuint handle, int width, int height, std::string name);
		virtual ~Framebuffer();

		void attachTexture(Attachment attachment, gltb::RefPtr<Texture> texture, Texture::CubeSide cubeSide=Texture::PositiveX);
		void attachTexture(Attachment attachment, int index, gltb::RefPtr<Texture> texture, Texture::CubeSide cubeSide=Texture::PositiveX);
		void attachTextureLayer(Attachment attachment, gltb::RefPtr<Texture> texture, int layer);
		void attachTextureLayer(Attachment attachment, int index, gltb::RefPtr<Texture> texture, int layer);
		bool isValid();

		virtual void setViewport(int x1, int y1, int x2, int y2);
		virtual void activate();
		virtual void deactivate();

		std::string getName() { return name; };
		int getWidth() { return width; };
		int getHeight() { return height; };

	private:
		void addAttachmentToActiveBuffers(GLenum attachment);

		GLuint handle;
		GLuint depthbufferHandle;
		int width;
		int height;
		int viewportX1;
		int viewportY1;
		int viewportX2;
		int viewportY2;
		std::vector<GLenum> activatedBuffers;
		bool isActive;

		std::string name;
	};
}

#endif /*FRAMEBUFFER_H_*/
