#ifndef TEXTURE_H
#define TEXTURE_H

#include  <string>

#include "GL/glew.h"

#include "GLTB/refptr.h"

#include "GL3Support/resource.h"

namespace GL3Support {
	class Framebuffer;

	class Texture : public Resource {
	public:
		enum Type {
			TwoD,
			TwoDRect,
			ThreeD,
			CubeMap,
			Buffer
		};

		enum Format {
			FloatR,
			FloatRG,
			FloatRGB,
			FloatRGBA,
			HalfR,
			HalfRG,
			HalfRGB,
			HalfRGBA,
			UCharR,
			UCharRI,
			UCharRG,
			UCharRGI,
			UCharRGB,
			UCharRGBI,
			UCharRGBA,
			UCharRGBAI,
			UCharBGR,
			UCharBGRI,
			UCharBGRA,
			UCharBGRAI,
			CharR,
			CharRI,
			CharRG,
			CharRGI,
			CharRGB,
			CharRGBI,
			CharRGBA,
			CharRGBAI,
			IntRI,
			IntRGI,
			IntRGBI,
			IntRGBAI,
			UIntRI,
			UIntRGI,
			UIntRGBI,
			UIntRGBAI,
			Depth,
			DepthFloat32
		};

		enum CubeSide {
			PositiveX = GL_TEXTURE_CUBE_MAP_POSITIVE_X,
			NegativeX = GL_TEXTURE_CUBE_MAP_NEGATIVE_X,
			PositiveY = GL_TEXTURE_CUBE_MAP_POSITIVE_Y,
			NegativeY = GL_TEXTURE_CUBE_MAP_NEGATIVE_Y,
			PositiveZ = GL_TEXTURE_CUBE_MAP_POSITIVE_Z,
			NegativeZ = GL_TEXTURE_CUBE_MAP_NEGATIVE_Z
		};

		enum Filter {
			Nearest = GL_NEAREST,
			Linear = GL_LINEAR
		};

		enum WrapMode {
			Clamp = GL_CLAMP,
			Repeat = GL_REPEAT
		};

		Texture(Type type, Format format, int width, int height, std::string name, void *data=0);
		virtual ~Texture();

		void bind(int textureUnit);
		void unbind();

		void setFilter(Filter filter);
		void setWrapMode(WrapMode wrapMode);

		void generateMipmap();

		int getWidth() { return width; };
		int getHeight() { return height; };
		Type getType() { return type; };
		Format getFormat() { return format; };

		void readBack(int regionX=-1, int regionY=-1, int regionWidth=-1, int regionHeight=-1);
		void reupload();
		void *getData() { return data; };

		unsigned int getTextureID() { return textureID; };
		int getGLFormat() { return glFormat; };
		int getGLType() { return glType; };

		std::string getName() { return name; };

		virtual Resource::Type getResourceType() { return Resource::TextureResource; };
		virtual void reload() { /* TODO */ };

	private:
		bool fromFile;
		std::string filename;

		unsigned int textureID;
		Type type;
		Format format;
		int glType;
		int glFormat;
		int hostFormat;
		int dataType;
		int width;
		int height;
		int depth;
		int imageSize;
		unsigned int textureBufferID;

		gltb::RefPtr<Framebuffer> readBackFB;
		char *data;

		int textureUnit;

		std::string name;

		static int activeTextureUnit;
	};
};

#endif
