#ifndef VERTEXBUFFER_H
#define VERTEXBUFFER_H

#include <string>
#include <vector>

#include "GL3Support/resource.h"
#include "GL3Support/state.h"

namespace GL3Support {
	class VertexBuffer : public Resource {
	public:
		enum ExpectedUsage {
			Static = GL_STATIC_DRAW,
			Dynamic = GL_DYNAMIC_DRAW,
			Streamed = GL_STREAM_DRAW
		};

		enum PrimitiveType {
			Points,
			Lines,
			LineStrip,
			Triangles,
			TriangleStrip,
			Quads
		};

		enum ElementSemantics {
			Index,
			Attribute
		};

		enum ElementType {
			Char,
			Int,
			Float,
			Vec2,
			Vec3,
			Vec4,
			IVec2,
			IVec3,
			IVec4
		};

		struct ElementDescription {
			std::string name;
			ElementType type;
			ElementSemantics semantics;

			int offset;
			int stride;
			int count;
		};

		VertexBuffer(void *sourceBuffer, int size, std::vector<ElementDescription> format, ExpectedUsage expectedUsage=Static);
		~VertexBuffer();

		void setPrimitiveType(PrimitiveType type) { primitiveType=type; };
		PrimitiveType getPrimitiveType() { return primitiveType; };

		void setState(gltb::RefPtr<State> state) { this->state=state; vertexArraySetup=false; };
		gltb::RefPtr<State> getState() { return state; };

		void *getBuffer();
		void commitBuffer();

		void render(int first=0, int count=-1);

		virtual Type getResourceType() { return Resource::VertexBufferResource; };
		virtual void reload() { /* TODO */ };

	private:
		GLuint vertexBufferID;
		GLuint vertexArrayID;

		PrimitiveType primitiveType;
		gltb::RefPtr<State> state;

		void *data;
		int size;
		bool bufferOpened;

		std::vector<ElementDescription> bufferFormat;
		bool indexed;
		int indexCount;
		int indexOffset;
		int vertexCount;

		bool vertexArraySetup;

		void validateFormat();
		void setupVertexArray();
	};
}

#endif
