#include "glm/glm.hpp"
#include "glm/ext.hpp"

#include "GLTB/referencedobject.h"

#include "GL3Support/vertexbuffer.h"

namespace GL3Support {
	class VertexBufferBuilder : public gltb::ReferencedObject {
	public:
		VertexBufferBuilder(VertexBuffer::PrimitiveType primitiveType);

		void setVertexAttributeName(std::string name);
		void setNormalAttributeName(std::string name);
		void setColourAttributeName(std::string name);
		void setUVCoordinatesAttributeName(std::string name);

		void vertex(glm::vec3 vertex);
		void vertex(float x, float y, float z) { vertex(glm::vec3(x,y,z)); };
		void normal(glm::vec3 normal);
		void normal(float x, float y, float z) { normal(glm::vec3(x,y,z)); };
		void colour(glm::vec4 colour);
		void colour(float r, float g, float b, float a) { colour(glm::vec4(r,g,b,a)); };
		void colour(float r, float g, float b) { colour(glm::vec4(r,g,b,1)); };
		void uvCoordinate(glm::vec2 uvCoordinate);
		void uvCoordinate(float x, float y) { uvCoordinate(glm::vec2(x,y)); };

		gltb::RefPtr<VertexBuffer> generateVertexBuffer(VertexBuffer::ExpectedUsage expectedUsage=VertexBuffer::Static);

	private:
		VertexBuffer::PrimitiveType primitiveType;

		bool hasNormals;
		glm::vec3 currentNormal;
		bool hasColours;
		glm::vec4 currentColour;
		bool hasUVCoordinates;
		glm::vec2 currentUVCoordinate;
		
		std::string vertexAttributeName;
		std::string normalAttributeName;
		std::string colourAttributeName;
		std::string uvCoordinatesAttributeName;

		std::vector<glm::vec3> vertices;
		std::vector<glm::vec3> normals;
		std::vector<glm::vec4> colours;
		std::vector<glm::vec2> uvCoordinates;
	};
}
