#include <cmath>

#include "GL/glew.h"
#include "GL/glu.h"

#include "GLTB/exception.h"
#include "GLTB/mathfunctions.h"

#include "GL3Support/camera.h"

namespace GL3Support {
	Camera::Camera()
	{
		// set some kind of default perspective projection
		setPerspectiveProjection(90,640/480,1,1000);
	}

	Camera::~Camera()
	{
	}

	void Camera::setPerspectiveProjection(float fieldOfView, float aspectRatio, float nearClip, float farClip)
	{
		this->fieldOfView=fieldOfView;
		this->aspectRatio=aspectRatio;
		this->nearClip=nearClip;
		this->farClip=farClip;
		projectionType=Perspective;

#if GLM_FORCE_RADIANS
		float fieldOfViewRad=fieldOfView*float(M_PI/180.0);
		projectionMatrix=glm::perspective(fieldOfViewRad,aspectRatio,nearClip,farClip);
#else
		projectionMatrix=glm::perspective(fieldOfView,aspectRatio,nearClip,farClip);
#endif
	}

	void Camera::getPerspectiveProjection(float &fieldOfView, float &aspectRatio, float &nearClip, float &farClip)
	{
		fieldOfView=this->fieldOfView;
		aspectRatio=this->aspectRatio;
		nearClip=this->nearClip;
		farClip=this->farClip;
	}

	void Camera::setOrthographicProjection(float left, float right, float bottom, float top, float near, float far)
	{
		this->left=left;
		this->right=right;
		this->bottom=bottom;
		this->top=top;
		this->near=near;
		this->far=far;
		projectionType=Orthographic;

		projectionMatrix=glm::ortho(left,right,bottom,top,near,far);
	}

	void Camera::getOrthographicProjection(float &left, float &right, float &bottom, float &top, float &near, float &far)
	{
		left=this->left;
		right=this->right;
		bottom=this->bottom;
		top=this->top;
		near=this->near;
		far=this->far;
	}

	Camera::ProjectionType Camera::getProjectionType()
	{
		return projectionType;
	}

	glm::mat4 Camera::getProjectionMatrix()
	{
		return projectionMatrix;
	}

	void Camera::setPositionLookAt(glm::vec3 position, glm::vec3 front, glm::vec3 up)
	{
		transformationMatrix=glm::lookAt(position,front,up);
	}

	void Camera::setTransformationMatrix(glm::mat4 matrix)
	{
		transformationMatrix=matrix;
	}

	glm::mat4 Camera::getTransformationMatrix()
	{
		return transformationMatrix;
	}

	glm::vec3 Camera::getFront()
	{
		glm::vec4 pos(0,0,0,1);
		glm::vec4 z(0,0,-1,1);

		glm::mat4 inverseTransform=glm::inverse(transformationMatrix);

		glm::vec4 front=inverseTransform*z-inverseTransform*pos;

		return glm::vec3(front.x,front.y,front.z);
	}

	glm::vec3 Camera::getUp()
	{
		glm::vec4 pos(0,0,0,1);
		glm::vec4 y(0,1,0,1);

		glm::mat4 inverseTransform=glm::inverse(transformationMatrix);

		glm::vec4 up=inverseTransform*y-inverseTransform*pos;
		return glm::vec3(up.x,up.y,up.z);
	}
}
