#include "GL3Support/fpscamera.h"

namespace GL3Support {
	FPSCamera::FPSCamera()
	{
		upAxis=YAxis;
		position=glm::vec3(0,0,0);
		yaw=0;
		pitch=0;
		roll=0;
	}

	FPSCamera::~FPSCamera()
	{
	}

	void FPSCamera::moveForward(double amount)
	{
		glm::vec3 forward(0.0,0.0,-amount);
		move(forward);
	}

	void FPSCamera::moveBackward(double amount)
	{
		glm::vec3 backward(0.0,0.0,amount);
		move(backward);
	}

	void FPSCamera::moveLeft(double amount)
	{
		glm::vec3 left(-amount,0.0,0.0);
		move(left);
	}

	void FPSCamera::moveRight(double amount)
	{
		glm::vec3 right(amount,0.0,0.0);
		move(right);
	}

	void FPSCamera::moveUp(double amount)
	{
		glm::vec3 up(0.0,amount,0.0);
		move(up);
	}

	void FPSCamera::moveDown(double amount)
	{
		glm::vec3 down(0.0,-amount,0.0);
		move(down);
	}

	void FPSCamera::move(glm::vec3 direction)
	{
		//Quaternion rotation;
		//rotation.fromYawPitchRoll(yaw,pitch,roll);

		switch(upAxis) {
		case YAxis:
			{
				glm::quat yawQuaternion=glm::angleAxis(-yaw,glm::vec3(1.0,0.0,0.0));
				glm::quat pitchQuaternion=glm::angleAxis(-pitch,glm::vec3(0.0,1.0,0.0));
				glm::quat rollQuaternion=glm::angleAxis(-roll,glm::vec3(0.0,0.0,-1.0));

				direction=rollQuaternion*direction;
				direction=yawQuaternion*direction;
				direction=pitchQuaternion*direction;
			}
			break;
		case ZAxis:
			{
				glm::quat yawQuaternion=glm::angleAxis(-yaw,glm::vec3(1.0,0.0,0.0));
				glm::quat pitchQuaternion=glm::angleAxis(-pitch,glm::vec3(0.0,0.0,-1.0));
				glm::quat rollQuaternion=glm::angleAxis(-roll,glm::vec3(0.0,-1.0,0.0));

				direction=rollQuaternion*direction;
				direction=yawQuaternion*direction;
				direction=pitchQuaternion*direction;
			}
			break;
		}
		position+=direction;
		updateTransformation();
	}

	void FPSCamera::rotateLeftRight(float amount)
	{
		pitch-=amount;
		updateTransformation();
	}

	void FPSCamera::rotateUpDown(float amount)
	{
		yaw-=amount;
		updateTransformation();
	}

	void FPSCamera::setPosition(glm::vec3 position)
	{
		this->position=position;
		updateTransformation();
	}

	void FPSCamera::setPositionLookAt(glm::vec3 position, glm::vec3 front, glm::vec3 up)
	{
		this->position=position;
		Camera::setPositionLookAt(position,front,up);
		// FIXME get yaw, pitch and roll angles from front and up vector
	}

	void FPSCamera::updateTransformation()
	{
		glm::mat4 translationMatrix=glm::translate(position);

		glm::quat rotation;

		glm::mat4 rotationMatrix;
		switch(upAxis) {
		case YAxis:
			{
				glm::quat yawQuaternion=glm::angleAxis(yaw,glm::vec3(1.0,0.0,0.0));
				glm::quat pitchQuaternion=glm::angleAxis(pitch,glm::vec3(0.0,1.0,0.0));
				glm::quat rollQuaternion=glm::angleAxis(roll,glm::vec3(0.0,0.0,-1.0));
				rotationMatrix=glm::mat4_cast(pitchQuaternion)*glm::mat4_cast(yawQuaternion)*glm::mat4_cast(rollQuaternion);
			}
			break;
		case ZAxis:
			{
				glm::quat yawQuaternion=glm::angleAxis(yaw,glm::vec3(1.0,0.0,0.0));
				glm::quat pitchQuaternion=glm::angleAxis(pitch,glm::vec3(0.0,0.0,-1.0));
				glm::quat rollQuaternion=glm::angleAxis(roll,glm::vec3(0.0,-1.0,0.0));
				rotationMatrix=glm::mat4_cast(pitchQuaternion)*glm::mat4_cast(yawQuaternion)*glm::mat4_cast(rollQuaternion);
			}
			break;
		}

		setTransformationMatrix(translationMatrix*rotationMatrix);
	}
}
