/*
 * glhelper.cpp
 *
 *  Created on: Jun 17, 2009
 *      Author: muecklgr
 */

#include <iostream>
#include <cstring>

#include "GL/glew.h"

#include "GL3Support/glhelper.h"

namespace GL3Support {
	namespace GLHelper {
		void checkGLError()
		{
			while(true) {
				int error=glGetError();

				if(error==GL_NO_ERROR) {
					return;
				}

				switch(error) {
				case GL_INVALID_ENUM:
					std::cerr << "GL error: invalid enum" << std::endl;
					break;
				case GL_INVALID_VALUE:
					std::cerr << "GL error: invalid value" << std::endl;
					break;
				case GL_INVALID_OPERATION:
					std::cerr << "GL error: invalid operation" << std::endl;
					break;
				case GL_INVALID_FRAMEBUFFER_OPERATION:
					std::cerr << "GL error: invalid framebuffer operation" << std::endl;
					break;
				case GL_STACK_OVERFLOW:
					std::cerr << "GL error: stack overflow" << std::endl;
					break;
				case GL_STACK_UNDERFLOW:
					std::cerr << "GL error: stack underflow" << std::endl;
					break;
				case GL_OUT_OF_MEMORY:
					std::cerr << "GL error: out of memory" << std::endl;
					break;
				case GL_TABLE_TOO_LARGE:
					std::cerr << "GL error: table to large" << std::endl;
					break;
				}
			}

		}



		void debugLogCallback(unsigned int source, unsigned int type, unsigned int id, unsigned int severity, int length, const char *message, void *params)
		{
			if (source == GL_DEBUG_SOURCE_API_ARB) {
				std::cout << "OpenGL\t\t";
			} else if (source == GL_DEBUG_SOURCE_WINDOW_SYSTEM_ARB) {
				std::cout << "Windowing System\t";
			} else if (source == GL_DEBUG_SOURCE_SHADER_COMPILER_ARB) {
				std::cout << "Shader Compiler\t";
			} else if (source == GL_DEBUG_SOURCE_THIRD_PARTY_ARB) {
				std::cout << "Third Party\t";
			} else if (source == GL_DEBUG_SOURCE_APPLICATION_ARB) {
				std::cout << "Application\t";
			} else if (source == GL_DEBUG_SOURCE_OTHER_ARB) {
				std::cout << "Other\t\t";
			} else {
				std::cout << "Unknown\t";
			}

			if (type == GL_DEBUG_TYPE_ERROR_ARB) {
				std::cout << "Error\t";
			} else if (type == GL_DEBUG_TYPE_DEPRECATED_BEHAVIOR_ARB) {
				std::cout << "Deprecated\t";
			} else if (type == GL_DEBUG_TYPE_UNDEFINED_BEHAVIOR_ARB) {
				std::cout << "Undefined\t";
			} else if (type == GL_DEBUG_TYPE_PORTABILITY_ARB) {
				std::cout << "Portability\t";
			} else if (type == GL_DEBUG_TYPE_PERFORMANCE_ARB) {
				std::cout << "Performance\t";
			} else if (type == GL_DEBUG_TYPE_OTHER_ARB) {
				std::cout << "Other\t";
			} else {
				std::cout << "Unknown\t";
			}

			if (severity == GL_DEBUG_SEVERITY_HIGH_ARB) {
				std::cout << "High\t";
			} else if (severity == GL_DEBUG_SEVERITY_MEDIUM_ARB) {
				std::cout << "Medium\t";
			} else if (severity == GL_DEBUG_SEVERITY_LOW_ARB) {
				std::cout << "Low\t";
			} else {
				std::cout << "Unknown\t";
			}
			std::cout << message << std::endl;
		}


		void enableDebugLogging()
		{
			if(!GLEW_ARB_debug_output) {
				std::cout << "WARNING: GL_ARB_debug_output is not supported/enabled - cannot enable debug logging" << std::endl;
				return;
			}

			glDebugMessageCallbackARB((GLDEBUGPROCARB)&debugLogCallback,0);
			glEnable(GL_DEBUG_OUTPUT);
			glDebugMessageControlARB(GL_DONT_CARE,GL_DONT_CARE,GL_DONT_CARE,0,0,GL_TRUE);

			const char *message="Test";
			glDebugMessageInsertARB(GL_DEBUG_SOURCE_APPLICATION_ARB,GL_DEBUG_TYPE_OTHER_ARB,10,GL_DEBUG_SEVERITY_HIGH_ARB,strlen(message),message);
			//glDebugMessageInsertARB(GL_DEBUG_SOURCE_API_ARB,GL_DEBUG_TYPE_OTHER_ARB,10,GL_DEBUG_SEVERITY_HIGH_ARB,strlen(message),message);
		}
	}
}
