#ifdef USE_DEVIL
#include "IL/il.h"
#endif

#include "GLTB/exception.h"

#include "GL3Support/resourcemanager.h"

namespace GL3Support {
	ResourceManager *ResourceManager::instance=0;

	ResourceManager *ResourceManager::getInstance()
	{
		if(instance==0) {
			instance=new ResourceManager();
		}
		return instance;
	}

	gltb::RefPtr<Shader> ResourceManager::createShader()
	{
		gltb::RefPtr<Shader> shader=new Shader();
		resources.push_back(gltb::RefPtr<Resource>(shader));
		return shader;
	}

	gltb::RefPtr<Texture> ResourceManager::createTexture(Texture::Type type, Texture::Format format, int width, int height, std::string name, void *data)
	{
		gltb::RefPtr<Texture> texture=new Texture(type,format,width,height,name,data);
		resources.push_back(gltb::RefPtr<Resource>(texture));
		return texture;
	}

#ifdef USE_DEVIL
	gltb::RefPtr<Texture> ResourceManager::loadTexture(std::string filename)
	{
		gltb::RefPtr<Texture> texture=getTexture(filename);
		if(texture!=0) {
			return texture;
		}

		ilInit();
		ILuint image=ilGenImage();

		ilBindImage(image);
		if(!ilLoadImage(filename.c_str())) {
			throw gltb::Exception("Unable to load texture from file " + filename,"GL3Support::ResourceManager::loadTexture()");
		}

		int width=ilGetInteger(IL_IMAGE_WIDTH);
		int height=ilGetInteger(IL_IMAGE_HEIGHT);
		int type=ilGetInteger(IL_IMAGE_TYPE);
		int format=ilGetInteger(IL_IMAGE_FORMAT);

		Texture::Format textureFormat;
		switch(type) {
		case IL_BYTE:
			switch(format) {
			case IL_LUMINANCE:
				textureFormat=Texture::CharR;
				break;
			case IL_RGB:
				textureFormat=Texture::CharRGB;
				break;
			case IL_RGBA:
				textureFormat=Texture::CharRGBA;
				break;
			default:
				throw gltb::Exception("Unsupported image data format in file " + filename,"GL3Support::ResourceManager::loadTexture()");
			}
			break;
		case IL_UNSIGNED_BYTE:
			switch(format) {
			case IL_LUMINANCE:
				textureFormat=Texture::UCharR;
				break;
			case IL_RGB:
				textureFormat=Texture::UCharRGB;
				break;
			case IL_RGBA:
				textureFormat=Texture::UCharRGBA;
				break;
			case IL_BGR:
				textureFormat=Texture::UCharBGR;
				break;
			case IL_BGRA:
				textureFormat=Texture::UCharBGRA;
				break;
			default:
				throw gltb::Exception("Unsupported image data format in file " + filename,"GL3Support::ResourceManager::loadTexture()");
			}
			break;
	//	case IL_SHORT:
	//		break;
	//	case IL_UNSIGNED_SHORT:
	//		break;
		case IL_INT:
			switch(format) {
			case IL_LUMINANCE:
				textureFormat=Texture::IntRI;
				break;
			case IL_RGB:
				textureFormat=Texture::IntRGBI;
				break;
	//		case IL_RGBA:
	//			textureFormat=Texture::IntRGBA;
	//			break;
			default:
				throw gltb::Exception("Unsupported image data format in file " + filename,"GL3Support::ResourceManager::loadTexture()");
			}
			break;
		case IL_UNSIGNED_INT:
			switch(format) {
			case IL_LUMINANCE:
				textureFormat=Texture::UIntRI;
				break;
			case IL_RGB:
				textureFormat=Texture::UIntRGBI;
				break;
	//		case IL_RGBA:
	//			textureFormat=Texture::UIntRGBA;
	//			break;
			default:
				throw gltb::Exception("Unsupported image data format in file " + filename,"GL3Support::ResourceManager::loadTexture()");
			}
			break;
		case IL_FLOAT:
			switch(format) {
			case IL_LUMINANCE:
				textureFormat=Texture::FloatR;
				break;
			case IL_RGB:
				textureFormat=Texture::FloatRGB;
				break;
			case IL_RGBA:
				textureFormat=Texture::FloatRGBA;
				break;
			default:
				throw gltb::Exception("Unsupported image data format in file " + filename,"GL3Support::ResourceManager::loadTexture()");
			}
			break;
	//	case IL_DOUBLE:
	//		break;
	//	case IL_HALF:
	//		break;
		default:
			throw gltb::Exception("Unsupported image data format in file " + filename,"GL3Support::ResourceManager::loadTexture()");
		}

		void *data=ilGetData();
		texture=new Texture(Texture::TwoD,textureFormat,width,height,filename,data);

		ilDeleteImage(image);

		resources.push_back(texture);
		return texture;
	}
#endif

	gltb::RefPtr<VertexBuffer> ResourceManager::createVertexBuffer(void *sourceBuffer, int size, std::vector<VertexBuffer::ElementDescription> format)
	{
		gltb::RefPtr<VertexBuffer> vertexBuffer=new VertexBuffer(sourceBuffer,size,format);
		resources.push_back(gltb::RefPtr<Resource>(vertexBuffer));
		return vertexBuffer;
	}

	gltb::RefPtr<Texture> ResourceManager::getTexture(std::string textureName)
	{
		for(unsigned int i=0;i<resources.size();i++) {
			gltb::RefPtr<Resource> resource=resources[i].getRefPtr();
			if(resource!=0) {
				if(resource->getResourceType()==Resource::TextureResource) {
					gltb::RefPtr<Texture> texture=resource;
					if(texture->getName()==textureName) {
						return texture;
					}
				}
			}
		}

		return 0;
	}

	void ResourceManager::reloadAll()
	{
		// use this chance to also clear old/stale weak references from the array
		std::vector<gltb::WeakPtr<Resource> > newResources;
		for(unsigned int i=0;i<resources.size();i++) {
			gltb::RefPtr<Resource> resource=resources[i].getRefPtr();
			if(resource!=0) {
				resource->reload();
				newResources.push_back(resource);
			}
		}

		resources=newResources;
	}
}
