/*
 * statemanager.cpp
 *
 *  Created on: Jul 24, 2010
 *      Author: gregor
 */

#include <iostream>

#include <GL/glew.h>

#include "GL3Support/glhelper.h"
#include "GL3Support/statemanager.h"

namespace GL3Support {
	StateManager *StateManager::instance=0;

	StateManager *StateManager::getInstance()
	{
		if(instance==0) {
			instance=new StateManager();
		}
		return instance;
	}

	StateManager::StateManager()
	{
		currentState=new State();

		resetCurrentState();
	}

	void StateManager::updateDepthTest(gltb::RefPtr<State> state)
	{
//		std::cout << "updating depth test state" << std::endl;
		if(state->depthTestEnabled) {
			glEnable(GL_DEPTH_TEST);
		} else {
			glDisable(GL_DEPTH_TEST);
		}
		glDepthFunc(state->depthTest);
//		GLHelper::checkGLError();
	}

	void StateManager::updateBlending(gltb::RefPtr<State> state)
	{
//		std::cout << "updating blending state" << std::endl;
		if(state->blendingEnabled) {
			glEnable(GL_BLEND);
		} else {
			glDisable(GL_BLEND);
		}
		glBlendFunc(state->sourceBlendFunction,state->destinationBlendFunction);
//		GLHelper::checkGLError();
	}

	void StateManager::updateFaceCulling(gltb::RefPtr<State> state)
	{
//		std::cout << "updating face culling state" << std::endl;
		if(state->faceCullingEnabled) {
			glEnable(GL_CULL_FACE);
		} else {
			glDisable(GL_CULL_FACE);
		}
		glCullFace(state->faceCulling);
//		GLHelper::checkGLError();
	}

	void StateManager::updateTexturing(gltb::RefPtr<State> state)
	{
		if(state->texture2DEnabled) {
			glEnable(GL_TEXTURE_2D);
		} else {
			glDisable(GL_TEXTURE_2D);
		}
		if(state->texture2DRectEnabled) {
			glEnable(GL_TEXTURE_RECTANGLE_ARB);
		} else {
			glDisable(GL_TEXTURE_RECTANGLE_ARB);
		}
		if(state->texture3DEnabled) {
			glEnable(GL_TEXTURE_3D);
		} else {
			glDisable(GL_TEXTURE_3D);
		}
		if(state->textureCubeEnabled) {
			glEnable(GL_TEXTURE_CUBE_MAP);
		} else {
			glDisable(GL_TEXTURE_CUBE_MAP);
		}
	}

	void StateManager::updateShader(gltb::RefPtr<State> state)
	{
//		std::cout << "updating shader state" << std::endl;
		if(currentState->shader!=0) {
			currentState->shader->deactivate();
		}
		if(state->shader!=0) {
			state->shader->activate();
		}
//		GLHelper::checkGLError();
	}

	void StateManager::setState(gltb::RefPtr<State> state)
	{
		if(state->depthTestEnabled!=currentState->depthTestEnabled ||
				state->depthTest!=currentState->depthTest) {
			updateDepthTest(state);
		}
		if(state->blendingEnabled!=currentState->blendingEnabled ||
				state->sourceBlendFunction!=currentState->sourceBlendFunction ||
				state->destinationBlendFunction!=currentState->destinationBlendFunction) {
			updateBlending(state);
		}
		if(state->faceCullingEnabled!=currentState->faceCullingEnabled ||
				state->faceCulling!=currentState->faceCulling) {
			updateFaceCulling(state);
		}
		if(state->texture2DEnabled!=currentState->texture2DEnabled || state->texture2DRectEnabled!=currentState->texture2DRectEnabled ||
				state->texture3DEnabled!=currentState->texture3DEnabled || state->textureCubeEnabled!=currentState->textureCubeEnabled) {
			updateTexturing(state);
		}
		if(state->shader!=currentState->shader || currentState->shader->hasChangedUniforms()) {
			updateShader(state);
		}

		currentState=state;
	}

	void StateManager::resetCurrentState()
	{
		updateDepthTest(currentState);
		updateBlending(currentState);
		updateFaceCulling(currentState);
		updateTexturing(currentState);
		updateShader(currentState);
	}
}
